// $Id: MainWindow.cpp,v 1.46 2014/01/16 19:55:27 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"
#include "MainWindow.moc"

#include "Application.h"
#include "BaseStatusBar.h"
#include "Color.h"
#include "ContextMenu.h"
#include "CustomToolBar.h"
#include "IconEngine.h"
#include "Icons.h"
#include "Menu.h"
#include "QtUtil.h"
#include "ServerXmlDef.h"
#include "Singleton.h"
#include "TreeView.h"
#include "XmlOptions.h"

#include <QLayout>

//___________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    verbosity_( LOW )

{
    setOptionName( "MAIN_WINDOW" );
    Debug::Throw( "MainWindow::MainWindow.\n" );

    // install actions
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    main->setLayout(  new QVBoxLayout() );
    main->layout()->setMargin(0);
    main->layout()->setSpacing(5);
    setCentralWidget( main );

    // toolbar
    toolbar_ = new CustomToolBar( tr( "Main" ), this, "TOOLBAR" );
    toolBar().addAction( &resetAction() );
    toolBar().addAction( &unlockAction() );
    toolBar().addSeparator();
    toolBar().addAction( &raiseAction() );
    toolBar().addAction( &abortAction() );


    // command list
    main->layout()->addWidget( list_ = new TreeView( main ) );
    list_->setModel( &model_ );
    list_->setSortingEnabled( true );
    list_->setRootIsDecorated( true );
    list_->setSelectionMode( TreeView::ContiguousSelection );
    list_->setOptionName( "LIST" );

    QMenu* menu( new ContextMenu( list_ ) );
    menu->addAction( &resetAction() );
    menu->addAction( &unlockAction() );
    menu->addSeparator();
    menu->addAction( &raiseAction() );
    menu->addAction( &abortAction() );
    menu->addSeparator();
    menu->addAction( &toolBar().visibilityAction() );

    connect( list_->selectionModel(), SIGNAL(selectionChanged (QItemSelection,QItemSelection)), SLOT(_updateActions()) );
    connect( list_, SIGNAL(activated(QModelIndex)), SLOT(_requestRaise()) );

    // status bar
    setStatusBar( statusBar_ = new BaseStatusBar( this ) );
    statusBar_->addLabel( 1, true );
    statusBar_->addClock();

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );

    connect( &application, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();
    _updateActions();

}

//_______________________________________
void MainWindow::append( const ServerCommandModel::List& commands )
{

    Debug::Throw( "MainWindow::append.\n" );

    // update state frame
    ServerCommandModel::List addedCommands;
    ServerCommandModel::List removedCommands;
    foreach( const SERVER::ServerCommand& command, commands )
    {

        statusBar_->label().setTextAndUpdate( QString( tr( "Recieved %1 from %2" ) ).arg( command.commandName() ).arg( command.id().name() ) );

        if( verbosity_ == LOW && (
            command.command() == SERVER::ServerCommand::Raise ||
            command.command() == SERVER::ServerCommand::Alive ||
            command.command() == SERVER::ServerCommand::Denied ) )
        { continue; }

        // for KILLED command and low verbosity, removes corresponding item
        if( verbosity_ == LOW && command.command() == SERVER::ServerCommand::Killed )
        {

            ServerCommandModel::List commands( model_.children() );
            for( ServerCommandModel::List::iterator iter = commands.begin(); iter != commands.end(); ++iter )
            { if( iter->id() == command.id() ) removedCommands << *iter; }
            continue;

        }

        addedCommands << command;
    }

    model_.remove( removedCommands );
    model_.add( addedCommands );

    list_->resizeColumnToContents( ServerCommandModel::Name );

    return;
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    event->accept();
}

//___________________________________________________________
void MainWindow::_updateConfiguration( void )
{

    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // resize
    setVerbosity( (Verbosity) XmlOptions::get().get<int>( "VERBOSITY" ) );

}

//_______________________________________
void MainWindow::_updateActions( void )
{

    Debug::Throw( "MainWindow::_updateActions.\n" );
    bool hasSelection( !list_->selectionModel()->selectedIndexes().empty() );
    raiseAction().setEnabled( hasSelection );
    abortAction().setEnabled( hasSelection );
    return;
}

//________________________________________
void MainWindow::_requestReset( void )
{
    Debug::Throw( "MainWindow::_requestReset.\n" );
    statusBar_->label().setTextAndUpdate( tr( "Resetting list..." ) );
    model_.clear();
    emit requestCommand( SERVER::ServerCommand::Identify );
}

//________________________________________
void MainWindow::_requestUnlock( void )
{
    Debug::Throw( "MainWindow::_requestUnlock.\n" );
    statusBar_->label().setTextAndUpdate( tr( "Unlocking registered applications..." ) );
    model_.clear();
    emit requestCommand( SERVER::ServerCommand::Unlock );
    emit requestCommand( SERVER::ServerCommand::Identify );
}

//________________________________________
void MainWindow::_requestRaise( void )
{
    Debug::Throw( "MainWindow::_requestRaise.\n" );
    ServerCommandModel::List commands( model_.get( list_->selectionModel()->selectedRows() ) );
    for( ServerCommandModel::List::iterator iter = commands.begin(); iter != commands.end(); ++iter )
    { emit requestCommand( SERVER::ServerCommand( iter->id(), SERVER::ServerCommand::Raise ) ); }
}

//________________________________________
void MainWindow::_requestAbort( void )
{
    Debug::Throw( "MainWindow::_requestAbort.\n" );

    ServerCommandModel::List commands( model_.get( list_->selectionModel()->selectedRows() ) );
    for( ServerCommandModel::List::iterator iter = commands.begin(); iter != commands.end(); ++iter )
    { emit requestCommand( SERVER::ServerCommand( iter->id(), SERVER::ServerCommand::Abort ) ); }
    _requestReset();

}

//________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installAction.\n" );

    addAction( resetAction_ = new QAction( IconEngine::get( ICONS::RESET ), tr( "Reload" ), this ) );
    resetAction_->setShortcut( QKeySequence::Refresh );
    connect( resetAction_, SIGNAL(triggered()), SLOT(_requestReset()) );

    addAction( unlockAction_ = new QAction( IconEngine::get( ICONS::UNLOCK ), tr( "Unlock" ), this ) );
    connect( unlockAction_, SIGNAL(triggered()), SLOT(_requestUnlock()) );

    // actions
    addAction( raiseAction_ = new QAction( IconEngine::get( ICONS::FIND ), tr( "Raise" ), this ) );
    raiseAction_->setToolTip( tr( "Raise selected applications" ) );
    connect( raiseAction_, SIGNAL(triggered()), SLOT(_requestRaise()) );


    addAction( abortAction_ = new QAction( IconEngine::get( ICONS::ABORT ), tr( "Abort" ), this ) );
    abortAction_->setToolTip( tr( "Abort selected applications" ) );
    connect( abortAction_, SIGNAL(triggered()), SLOT(_requestAbort()) );

}
